@@ -24,7 +24,6 @@ import traceback
 import datetime
 from textwrap import TextWrapper
 from contextlib import contextmanager
-from blessings import Terminal
 
 import click
 from click import UsageError
@@ -532,7 +531,7 @@ class App():
                 queue = job.queue
 
                 # Get the last failure message for additional context
-                failure = self._fail_messages.get(element._get_unique_id())
+                failure = self._fail_messages.get(element._unique_id)
 
                 # XXX This is dangerous, sometimes we get the job completed *before*
                 # the failure message reaches us ??
