@@ -88,6 +88,7 @@ from ._variables import Variables
 from ._versions import BST_CORE_ARTIFACT_VERSION
 from ._exceptions import BstError, LoadError, LoadErrorReason, ImplError, ErrorDomain
 from .utils import UtilError
+from .types import _UniquePriorityQueue
 from . import Plugin, Consistency
 from . import SandboxFlags
 from . import utils
@@ -214,6 +215,8 @@ class Element(Plugin):
 
         self.__runtime_dependencies = []        # Direct runtime dependency Elements
         self.__build_dependencies = []          # Direct build dependency Elements
+        self.__reverse_dependencies = set()     # Direct reverse dependency Elements
+        self.__ready_for_runtime = False        # Wether the element has all its dependencies ready and has a cache key
         self.__sources = []                     # List of Sources
         self.__weak_cache_key = None            # Our cached weak cache key
         self.__strict_cache_key = None          # Our cached cache key for strict builds
@@ -434,7 +437,7 @@ class Element(Plugin):
                                                 visited=visited, recursed=True)
 
         # Yeild self only at the end, after anything needed has been traversed
-        if should_yield and (recurse or recursed) and (scope == Scope.ALL or scope == Scope.RUN):
+        if should_yield and (recurse or recursed) and scope != Scope.BUILD:
             yield self
 
     def search(self, scope, name):
@@ -685,6 +688,7 @@ class Element(Plugin):
         files_written = {}
         old_dep_keys = {}
         workspace = self._get_workspace()
+        project = self._get_project()
 
         if self.__can_build_incrementally() and workspace.last_successful:
             old_dep_keys = self.__get_artifact_metadata_dependencies(workspace.last_successful)
@@ -746,9 +750,8 @@ class Element(Plugin):
                 overlapping_elements = elements[1:]
                 for elm in overlapping_elements:
                     element = self.search(scope, elm)
-                    element_project = element._get_project()
                     if not element.__file_is_whitelisted(f):
-                        if element_project.fail_on_overlap:
+                        if project.fail_on_overlap:
                             overlap_error_elements.append(elm)
                             overlap_error = True
                         else:
@@ -924,9 +927,12 @@ class Element(Plugin):
         for meta_dep in meta.dependencies:
             dependency = Element._new_from_meta(meta_dep, artifacts)
             element.__runtime_dependencies.append(dependency)
+            dependency.__reverse_dependencies.add(element)
+
         for meta_dep in meta.build_dependencies:
             dependency = Element._new_from_meta(meta_dep, artifacts)
             element.__build_dependencies.append(dependency)
+            dependency.__reverse_dependencies.add(element)
 
         return element
 
@@ -1143,6 +1149,10 @@ class Element(Plugin):
                 # Strong cache key could not be calculated yet
                 return
 
+        if not self.__ready_for_runtime and self.__cache_key is not None:
+            self.__ready_for_runtime = all(
+                dep.__ready_for_runtime for dep in self.__runtime_dependencies)
+
     # _get_display_key():
     #
     # Returns cache keys for display purposes
@@ -1245,7 +1255,7 @@ class Element(Plugin):
         self.__tracking_scheduled = False
         self.__tracking_done = True
 
-        self._update_state()
+        self.__update_state_recursively()
 
     # _track():
     #
@@ -1262,7 +1272,7 @@ class Element(Plugin):
         for source in self.__sources:
             old_ref = source.get_ref()
             new_ref = source._track()
-            refs.append((source._get_unique_id(), new_ref))
+            refs.append((source._unique_id, new_ref))
 
             # Complimentary warning that the new ref will be unused.
             if old_ref != new_ref and self._get_workspace():
@@ -1289,17 +1299,21 @@ class Element(Plugin):
                 if scope == Scope.BUILD:
                     self.stage(sandbox)
                 elif scope == Scope.RUN:
-                    # Stage deps in the sandbox root
                     if deps == 'run':
-                        with self.timed_activity("Staging dependencies", silent_nested=True):
-                            self.stage_dependency_artifacts(sandbox, scope)
+                        dependency_scope = Scope.RUN
+                    else:
+                        dependency_scope = None
 
-                        # Run any integration commands provided by the dependencies
-                        # once they are all staged and ready
-                        if integrate:
-                            with self.timed_activity("Integrating sandbox"):
-                                for dep in self.dependencies(scope):
-                                    dep.integrate(sandbox)
+                    # Stage deps in the sandbox root
+                    with self.timed_activity("Staging dependencies", silent_nested=True):
+                        self.stage_dependency_artifacts(sandbox, dependency_scope)
+
+                    # Run any integration commands provided by the dependencies
+                    # once they are all staged and ready
+                    if integrate:
+                        with self.timed_activity("Integrating sandbox"):
+                            for dep in self.dependencies(dependency_scope):
+                                dep.integrate(sandbox)
 
             yield sandbox
 
@@ -1417,7 +1431,7 @@ class Element(Plugin):
         self.__assemble_scheduled = False
         self.__assemble_done = True
 
-        self._update_state()
+        self.__update_state_recursively()
 
         if self._get_workspace() and self._cached():
             #
@@ -1588,6 +1602,15 @@ class Element(Plugin):
 
         return artifact_size
 
+    # _fetch_done()
+    #
+    # Indicates that fetching the sources for this element has been done.
+    #
+    def _fetch_done(self):
+        # We are not updating the state recursively here since fetching can
+        # never end up in updating them.
+        self._update_state()
+
     # _pull_pending()
     #
     # Check whether the artifact will be pulled.
@@ -1621,7 +1644,7 @@ class Element(Plugin):
     def _pull_done(self):
         self.__pull_done = True
 
-        self._update_state()
+        self.__update_state_recursively()
 
     def _pull_strong(self, *, progress=None):
         weak_key = self._get_cache_key(strength=_KeyStrength.WEAK)
@@ -2500,6 +2523,24 @@ class Element(Plugin):
 
         return utils._deduplicate(keys)
 
+    # __update_state_recursively()
+    #
+    # Update the state of all reverse dependencies, recursively.
+    #
+    def __update_state_recursively(self):
+        queue = _UniquePriorityQueue()
+        queue.push(self._unique_id, self)
+
+        while queue:
+            element = queue.pop()
+
+            old_ready_for_runtime = element.__ready_for_runtime
+            element._update_state()
+
+            if element.__ready_for_runtime != old_ready_for_runtime:
+                for rdep in element.__reverse_dependencies:
+                    queue.push(rdep._unique_id, rdep)
+
 
 def _overlap_error_detail(f, forbidden_overlap_elements, elements):
     if forbidden_overlap_elements:
