@@ -26,6 +26,26 @@ def test_missing_project_name(cli, datafiles):
     result.assert_main_error(ErrorDomain.LOAD, LoadErrorReason.INVALID_DATA)
 
 
+@pytest.mark.datafiles(os.path.join(DATA_DIR))
+def test_missing_element(cli, datafiles):
+    project = os.path.join(datafiles.dirname, datafiles.basename, "missing-element")
+    result = cli.run(project=project, args=['show', 'manual.bst'])
+    result.assert_main_error(ErrorDomain.LOAD, LoadErrorReason.MISSING_FILE)
+
+    # Assert that we have the expected provenance encoded into the error
+    assert "manual.bst [line 4 column 2]" in result.stderr
+
+
+@pytest.mark.datafiles(os.path.join(DATA_DIR))
+def test_missing_junction(cli, datafiles):
+    project = os.path.join(datafiles.dirname, datafiles.basename, "missing-junction")
+    result = cli.run(project=project, args=['show', 'manual.bst'])
+    result.assert_main_error(ErrorDomain.LOAD, LoadErrorReason.MISSING_FILE)
+
+    # Assert that we have the expected provenance encoded into the error
+    assert "manual.bst [line 4 column 2]" in result.stderr
+
+
 @pytest.mark.datafiles(os.path.join(DATA_DIR))
 def test_empty_project_name(cli, datafiles):
     project = os.path.join(datafiles.dirname, datafiles.basename, "emptyname")
@@ -181,3 +201,15 @@ def test_project_refs_options(cli, datafiles):
 
     # Assert that the cache keys are different
     assert result1.output != result2.output
+
+
+@pytest.mark.datafiles(os.path.join(DATA_DIR, 'element-path'))
+def test_element_path_project_path_contains_symlinks(cli, datafiles, tmpdir):
+    real_project = str(datafiles)
+    linked_project = os.path.join(str(tmpdir), 'linked')
+    os.symlink(real_project, linked_project)
+    os.makedirs(os.path.join(real_project, 'elements'), exist_ok=True)
+    with open(os.path.join(real_project, 'elements', 'element.bst'), 'w') as f:
+        f.write("kind: manual\n")
+    result = cli.run(project=linked_project, args=['show', 'element.bst'])
+    result.assert_success()
