@@ -8,7 +8,6 @@ from collections import namedtuple
 
 from contextlib import contextmanager
 from multiprocessing import Process, Queue
-import pytest_cov
 
 from buildstream import _yaml
 from buildstream._artifactcache.cascache import CASCache
@@ -29,7 +28,11 @@ from buildstream._exceptions import ArtifactError
 #
 class ArtifactShare():
 
-    def __init__(self, directory, *, total_space=None, free_space=None):
+    def __init__(self, directory, *,
+                 total_space=None,
+                 free_space=None,
+                 min_head_size=int(2e9),
+                 max_head_size=int(10e9)):
 
         # The working directory for the artifact share (in case it
         # needs to do something outside of it's backend's storage folder).
@@ -53,6 +56,9 @@ class ArtifactShare():
         self.total_space = total_space
         self.free_space = free_space
 
+        self.max_head_size = max_head_size
+        self.min_head_size = min_head_size
+
         q = Queue()
 
         self.process = Process(target=self.run, args=(q,))
@@ -68,7 +74,13 @@ class ArtifactShare():
     # Run the artifact server.
     #
     def run(self, q):
-        pytest_cov.embed.cleanup_on_sigterm()
+
+        try:
+            import pytest_cov
+        except ImportError:
+            pass
+        else:
+            pytest_cov.embed.cleanup_on_sigterm()
 
         # Optionally mock statvfs
         if self.total_space:
@@ -76,7 +88,10 @@ class ArtifactShare():
                 self.free_space = self.total_space
             os.statvfs = self._mock_statvfs
 
-        server = create_server(self.repodir, enable_push=True)
+        server = create_server(self.repodir,
+                               max_head_size=self.max_head_size,
+                               min_head_size=self.min_head_size,
+                               enable_push=True)
         port = server.add_insecure_port('localhost:0')
 
         server.start()
@@ -118,6 +133,15 @@ class ArtifactShare():
 
         try:
             tree = self.cas.resolve_ref(artifact_key)
+            reachable = set()
+            try:
+                self.cas._reachable_refs_dir(reachable, tree, update_mtime=False)
+            except FileNotFoundError:
+                return False
+            for digest in reachable:
+                object_name = os.path.join(self.cas.casdir, 'objects', digest[:2], digest[2:])
+                if not os.path.exists(object_name):
+                    return False
             return True
         except ArtifactError:
             return False
@@ -149,8 +173,11 @@ class ArtifactShare():
 # Create an ArtifactShare for use in a test case
 #
 @contextmanager
-def create_artifact_share(directory, *, total_space=None, free_space=None):
-    share = ArtifactShare(directory, total_space=total_space, free_space=free_space)
+def create_artifact_share(directory, *, total_space=None, free_space=None,
+                          min_head_size=int(2e9),
+                          max_head_size=int(10e9)):
+    share = ArtifactShare(directory, total_space=total_space, free_space=free_space,
+                          min_head_size=min_head_size, max_head_size=max_head_size)
     try:
         yield share
     finally:
